; -----------------------------------------
; CPP9500 Spektrumanalyzer
; -----------------------------------------

	.device	ATmega2561

	.include "ram.inc"
	.include "m2561def.inc"
	.include "allgdef.inc"

	.cseg

        	jmp	start
	
	.org	0x2e
	jmp		timer0int

	.org	0x100


start:

; RAM lschen
	ldi	zh,high(0x200)
	ldi	zl,low(0x200)
	ldi	acc2,high(clrende)
	ldi	acc3,low(clrende)
	clr	acc
ramclr:
	st	z+,acc
	cp	acc3,zl
	cpc	acc2,zh
	brcc	ramclr
	
; Stack
	ldi	r16,low(ramend)
	out	spl,r16
	ldi	r16,high(ramend)
	out	sph,r16

; Ports einstellen
	ldi	r16,0b11111111
	out	ddra,r16
	ldi	r16,0b11111111	; A = PP-Ausgnge
	out	porta,r16
	
	ldi	r16,0b00011111
	out	ddrb,r16		; B = Inp + PP
	ldi	r16,0b11101000
	out	portb,r16
	
	ldi	r16,0b00000000
	out	ddrc,r16		; C = Inp
	ldi	r16,0b11111111
	out	portc,r16
	
	ldi	r16,0b11111111
	out	ddrd,r16		; D = PP
	ldi	r16,0b11100000
	out	portd,r16
	
	ldi	r16,0b11111111
	out	ddre,r16		; E = PP
	ldi	r16,0b11111111    
	out	porte,r16
	
	ldi	r16,0b01000000	; F = Ana + JTAG
	out	ddrf,r16
	ldi	r16,0b11111110
	out	portf,r16
	
	ldi	r16,0b11111111
	out	ddrg,r16		; G = PP
	ldi	r16,0b00000000
	out	portg,r16
	

; Timer0, Takt:
; TCCR0B (S.132) =0000$0101 = SysClock /1024 = 64us
; oder 0000$0010 = SysClock/8 mal 256(Timer-berlauf) = 128us
; oder 0000$0010 = SysClock/8 mal 200(Timer-Ladewert) = 100us
;
; Timer0, Interrupt
; TIMSK0 (S.134) = 0000$0001
	ldi		acc,0b00000010
	out		TCCR0B,acc
	ldi		acc,0b00000001
	sts		TIMSK0,acc
	sei

; Anfangsdaten

	call	restart
	call	lcdinit

;--------------------------------------------------
; HP  *********************************************
;--------------------------------------------------

hp:
	call	monitor
	call	tp
	call	lcdsnd
	call	tp
	call	rotorsw
	call	tp
	call	input
	call	tp
	call	timer
	call	tp
	call	edit
	call	tp
	call	analyzer
	call	tp
	call	relcon
	call	tp
	call	eeprom
	call	tp
	call	kalibrieren
	jmp		hp

tp:
	lds	acc,0xa00
	lds	acc2,testzsp4
	cp	acc,acc2
	breq	tp1
	nop
	nop
	nop
tp1:
	sts	testzsp4,acc
	ret
	
; ------------------------------------------------
; Interrupt
; ------------------------------------------------
; Timer0 OVF alle 128us berlauf-Int
; xl, xh, xzgr und xyacc = nur hier verwenden!
timer0int:
	push	acc
; Timer fr den Analyzer
	in		acc,sreg
	inc		softtimer
	lds		xyacc,ddstimer
	tst		xyacc
	breq	tim0intsp
	dec		xyacc
	sts		ddstimer,xyacc
; Pegel ausgeben fr den Sender
tim0intsp:
	inc		amfrequenz
	sbrc	amfrequenz,2
	rjmp	tim0sp
	lds		xyacc,pegel1
	out		porta,xyacc
	cbi		portd,pdawcs 		; Pegel Ausgabe
	sbi		portd,pdawcs
	rjmp	tim0int0
tim0sp:
	lds		xyacc,pegel2
	out		porta,xyacc
	cbi		portd,pdawcs 		; Pegel Ausgabe
	sbi		portd,pdawcs
		
; X Y - Ausgabe
tim0int0:
	sbrc	merker1,crsmod
	rjmp	tim0int2
	out		porta,xzgr
	ldi		xh,high(messpegel)
	mov		xl,xzgr
	ld		xyacc,x
	cbi		portd,xdawcs 		; X Ausgabe
	sbi		portd,xdawcs
	out		porta,xyacc
	cbi		portd,ydawcs    	; Y Ausgabe
	sbi		portd,ydawcs
	inc		xzgr
	breq	tim0int1
	out		sreg,acc
	pop		acc
	reti
tim0int1:
	sbr		merker1,1<<(crsmod)
	out		sreg,acc
	pop		acc
	reti
; CRT-Cursor
tim0int2:
	lds		xyacc,crszgr
	out		porta,xyacc
	cbi		portd,xdawcs 		; Pos. Cursor Ausgabe
	sbi		portd,xdawcs
	out		porta,xzgr			; hilfsweise fr den Cursor
	cbi		portd,ydawcs    	; Y-Hhe Cursor Ausgabe
	sbi		portd,ydawcs
	subi	xzgr,(256-16)
	breq	tim0int3
	out		sreg,acc
	pop		acc
	reti
tim0int3:
	cbr		merker1,1<<(crsmod)
	out		sreg,acc
	pop		acc
	reti
	
; ------------------------------------------------
	
.include	"monitor.asm"
.include	"texte.asm"
.include	"lcdsnd.asm"	
.include 	"font.inc"
.include	"diverse.asm"
.include	"b20d6.asm"
.include 	"dbtabelle.asm"
.include	"daa.asm"
.include	"rotorsw.asm"
.include 	"input.asm"
.include	"timer.asm"
.include 	"edit.asm"
.include 	"testcon.asm"
.include	"restart.asm"
.include	"ring.asm"
.include	"pllsender.asm"
.include	"analyzer.asm"
.include	"relcon.asm"
.include	"ddssender.asm"
.include	"mul4x3.asm"
.include	"dbkorrtabelle.asm"
.include	"eeprom.asm"
.include	"kalibrieren.asm"

